"""Unit test for the X.509 handling class."""

import unittest
import x509


rootcert = """
-----BEGIN CERTIFICATE-----
MIIHCDCCBPCgAwIBAgIJAMK89k6UIaUuMA0GCSqGSIb3DQEBBQUAMIGoMQswCQYD
VQQGEwJDSDEQMA4GA1UECBMHU29sZXVyZTEOMAwGA1UEBxMFT2x0ZW4xGjAYBgNV
BAoTEUFuY2llbnQgU29sdXRpb25zMRwwGgYDVQQLExNQbGF0Zm9ybSBNYW5hZ2Vt
ZW50MRAwDgYDVQQDEwdSb290IENBMSswKQYJKoZIhvcNAQkBFhxhZG1pbnNAYW5j
aWVudC1zb2x1dGlvbnMuY29tMB4XDTExMDYwNDE5MjkwOVoXDTMxMDUzMDE5Mjkw
OVowgagxCzAJBgNVBAYTAkNIMRAwDgYDVQQIEwdTb2xldXJlMQ4wDAYDVQQHEwVP
bHRlbjEaMBgGA1UEChMRQW5jaWVudCBTb2x1dGlvbnMxHDAaBgNVBAsTE1BsYXRm
b3JtIE1hbmFnZW1lbnQxEDAOBgNVBAMTB1Jvb3QgQ0ExKzApBgkqhkiG9w0BCQEW
HGFkbWluc0BhbmNpZW50LXNvbHV0aW9ucy5jb20wggIiMA0GCSqGSIb3DQEBAQUA
A4ICDwAwggIKAoICAQDW5HF6ncML1poz1Bf2/HtWKXTMMvmpl/Z0kLbO1DzH7Tvl
ROIq5rmXGvczbDsSQ6gdEJ5z0th0jh09mIun2tlgtbVmEVEJfCP8AUQBinDulBht
B856azhMQh9pX6IiO5d6y/Bv7I2xZqGA7ypyCJglsSFbP3XzTqz0eHohzVSDF28U
tpCIQS/BhfXDPUZ+7JY+WTmSAnyUU/CcK8GS6xCIS0ylbytPLqDMg2ivt62aKnsN
pKsgiJNTd7ZMGkHrchc4Xtr0thdGpWJKllu5RLIS/splan7oTmMAVLU1yHUlMl/f
Nh6iH3ZihKqHLCKE+zsIXlf3mgalI6KoSRI/lrRlZF60POByTSf7alyluwtFw1mV
RL1A7KHuH1Em2bwti0m3GQr0M5llmjG8veGKx038MqloUlz9kE6wh9JlwpbFqrxT
wFCNAMyA3rw4tB2EFpmdGWXN5b9u10H9Ds2ux/EjR9gbfp3nK3jyNA5CoKD79nXu
+beyv8cT+5Wj8RLGh1MiQ2sMO14mYKQ7H8xlFMnlAri8qIHh6vq3+HXLqKRpOVxt
n5D15jbeaK+lGuF/4sBBH7u2qpFF0Kiz8bbXx06eyhuV20FJbInPZXqCyh2dkx0l
Pohk6nCD22qwpSrqpV5S367GoGw3YY1TDoOmtemnshZtmP1JKR3WWMfXlh4QGQID
AQABo4IBMTCCAS0wHQYDVR0OBBYEFEDqFiamLHx7JF3Py2ynkCw/ARcKMIHdBgNV
HSMEgdUwgdKAFEDqFiamLHx7JF3Py2ynkCw/ARcKoYGupIGrMIGoMQswCQYDVQQG
EwJDSDEQMA4GA1UECBMHU29sZXVyZTEOMAwGA1UEBxMFT2x0ZW4xGjAYBgNVBAoT
EUFuY2llbnQgU29sdXRpb25zMRwwGgYDVQQLExNQbGF0Zm9ybSBNYW5hZ2VtZW50
MRAwDgYDVQQDEwdSb290IENBMSswKQYJKoZIhvcNAQkBFhxhZG1pbnNAYW5jaWVu
dC1zb2x1dGlvbnMuY29tggkAwrz2TpQhpS4wDAYDVR0TBAUwAwEB/zALBgNVHQ8E
BAMCAQYwEQYJYIZIAYb4QgEBBAQDAgEGMA0GCSqGSIb3DQEBBQUAA4ICAQBcPOLk
0ZVEJFGr65uZRRWm5CGQVZFTs3Cd4drGhUT11xbY/3EvGeHMx2caByCB1txVV1NO
pFj317hXLh9Zdwb81Td9tOYdMMRuuqyY3cdX3apNERsZus47+ioVgUc8Hm55X4be
N2fIr5bo/WX6C9yRWeKDz/W3nKY/CwhkRi1rVNENxMfZi5vVzZfXFQSV16c2uFaD
TIPbWNx+ezpv1jYFo1GJLgIDHFIJQqQFj8WyVrUOgiIOOg8DeTr0lMwa+Knu9ljU
hUwS7ABvNk0hQtlR/e2Pqh435CDF+A1DCBbaD5+SmEarXIh6QO3ej0ymO0PJsePm
/B07z1vglatOI6cOFBWrxXRO7QptqQb7g+SVilZEjv7XYT7ZzFj9cv7jwx/kI8wl
ln8KSvuFi8U5XDDAm7bq7ONfLoy3Qp+lRO/fH2DWSvBtqsfXlf8KqHeIWbnwm3Er
WW400/Syg9aXr8B0zgql37TRCnBiyBiHT+hMSu3qP6wHdwX4vjtLoJUk+dAh64Vd
PHCUrDdxWBQY4vnSUMwLrjy2DZ9WjySJn8UbjFrDlWJQPQcK2ekd157lHxOFXFzf
yssJc1Z1aucECxxQPrq79odKql9alaf1MJwTGJm/56ao236qBBb2Mvxhsop5ryq8
1SGk8JDE8wEkjuj49UxfHg1Z1yrm0ksdIz3fBw==
-----END CERTIFICATE-----"""

subcert = """
-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----"""


class TestX509Functionality(unittest.TestCase):
    """Tests the functionality of the x509 class."""

    def test_check_signature(self):
        """Check if we can verify the signature of certificates."""

        rc = x509.parse_certificate(rootcert)
        sc = x509.parse_certificate(subcert)
        self.assertTrue(sc.check_signature(rc))


if __name__ == '__main__':
    unittest.main()
